@@ -1,7 +1,7 @@
-/* Conversion from and to ISO 8859-3.
-   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
+/* Conversion from and to ISO 8859-15.
+   Copyright (C) 1998 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.
+   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1998.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -20,9 +20,9 @@
 
 /* Get the conversion table.  */
 #include <stdint.h>
-#include <iso8859-3.h>
+#define TABLES <iso8859-15.h>
 
-#define CHARSET_NAME	"ISO-8859-3//"
-#define HAS_HOLES	1	/* Not all 256 character are defined.  */
+#define CHARSET_NAME	"ISO-8859-15//"
+#define HAS_HOLES	0	/* All 256 character are defined.  */
 
-#include <8bit-generic.c>
+#include <8bit-gap.c>
