@@ -3,6 +3,7 @@
 #include <sunrpc/rpc/auth_des.h>
 
 libc_hidden_proto (getpublickey)
+libc_hidden_proto (getsecretkey)
 libc_hidden_proto (rtime)
 
 extern bool_t xdr_authdes_cred (XDR *xdrs, struct authdes_cred *cred);
@@ -16,9 +17,9 @@ extern enum auth_stat _svcauth_des (register struct svc_req *rqst,
 
 #define DECLARE_NSS_PROTOTYPES(service)					\
 extern enum nss_status _nss_ ## service ## _getpublickey		\
-                       (const char *netname, char *pkey, int *errnop);	\
+		       (const char *netname, char *pkey, int *errnop);	\
 extern enum nss_status _nss_ ## service ## _getsecretkey		\
-                       (const char *netname, char *skey, char *passwd,	\
+		       (const char *netname, char *skey, char *passwd,	\
 			int *errnop);
 
 DECLARE_NSS_PROTOTYPES (files)
@@ -27,4 +28,8 @@ DECLARE_NSS_PROTOTYPES (nisplus)
 
 #undef DECLARE_NSS_PROTOTYPES
 
+libc_hidden_proto (authdes_getucred)
+libc_hidden_proto (xdr_authdes_cred)
+libc_hidden_proto (xdr_authdes_verf)
+
 #endif
