@@ -1,4 +1,4 @@
-/* Copyright (C) 1995-2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+/* Copyright (C) 1995-2011, 2012 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Ulrich Drepper <drepper@cygnus.com>, 1995.
 
@@ -388,7 +388,7 @@ print_version (FILE *stream, struct argp_state *state)
 Copyright (C) %s Free Software Foundation, Inc.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2011");
+"), "2012");
   fprintf (stream, gettext ("Written by %s.\n"), "Ulrich Drepper");
 }
 
@@ -417,7 +417,6 @@ construct_output_path (char *path)
 	 contains a reference to the codeset.  This should be
 	 normalized.  */
       char *startp;
-      size_t n;
 
       startp = path;
       /* We must be prepared for finding a CEN name or a location of
@@ -441,6 +440,7 @@ construct_output_path (char *path)
       /* We put an additional '\0' at the end of the string because at
 	 the end of the function we need another byte for the trailing
 	 '/'.  */
+      ssize_t n;
       if (normal == NULL)
 	n = asprintf (&result, "%s%s/%s%c",
 		      output_prefix ?: "", LOCALEDIR, path, '\0');
