@@ -25,7 +25,7 @@ static char rcsid[] = "$NetBSD: $";
  */
 
 #include <math.h>
-#include "math_private.h"
+#include <math_private.h>
 
 #ifdef __STDC__
 	long double __significandl(long double x)
